
-------------------------------------------------------------------------------
--
-- Game of Live/VGA raster display
--
-- Juan, Steve, Vinny - Team 24
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

entity vga_raster is
  
  port (
    reset : in std_logic;
    clk25_diff   : in std_logic;                    -- Should be 25.125 MHz

    signal avs_s1_clk,
    avs_s1_reset_n,
    avs_s1_read,
    avs_s1_write,
    avs_s1_chipselect : in std_logic;
    signal avs_s1_address : in std_logic_vector(10 downto 0);
    signal avs_s1_readdata : out std_logic_vector(31 downto 0);
    signal avs_s1_writedata : in std_logic_vector(31 downto 0);
    
    HEX : out std_logic_vector(6 downto 0);

    VGA_CLK,                                 -- Clock
    VGA_HS,                                  -- H_SYNC
    VGA_VS,                                  -- V_SYNC
    VGA_BLANK,                               -- BLANK
    VGA_SYNC : out std_logic;                -- SYNC
    VGA_R,                                   -- Red[9:0]
    VGA_G,                                   -- Green[9:0]
    VGA_B : out std_logic_vector(9 downto 0) -- Blue[9:0]
  );

end vga_raster;

architecture rtl of vga_raster is

  component board_mem
    PORT
      (
        address_a		: IN STD_LOGIC_VECTOR (10 DOWNTO 0);
        address_b		: IN STD_LOGIC_VECTOR (10 DOWNTO 0);
        clock		: IN STD_LOGIC ;
        data_a		: IN STD_LOGIC_VECTOR (31 DOWNTO 0);
        data_b		: IN STD_LOGIC_VECTOR (31 DOWNTO 0);
        wren_a		: IN STD_LOGIC  := '0';
        wren_b		: IN STD_LOGIC  := '0';
        q_a		: OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
        q_b		: OUT STD_LOGIC_VECTOR (31 DOWNTO 0)
        );
  end component;
  -- Video parameters
  
  constant HTOTAL       : integer := 800;
  constant HSYNC        : integer := 96;
  constant HBACK_PORCH  : integer := 48;
  constant HACTIVE      : integer := 640;
  constant HFRONT_PORCH : integer := 16;
  
  constant VTOTAL       : integer := 525;
  constant VSYNC        : integer := 2;
  constant VBACK_PORCH  : integer := 33;
  constant VACTIVE      : integer := 480;
  constant VFRONT_PORCH : integer := 10;
  
  --Hpixel_pos - starts at 0
  signal Hpixel_pos		: std_logic_vector(10 downto 0) := "00000000000";

  --Vpixel_pos - starts at 0
  signal Vpixel_pos		: std_logic_vector(10 downto 0) := "00000000000";

  -- Signals for the video controller
  signal Hcount : std_logic_vector(9 downto 0);  -- Horizontal position (0-800)
  signal Vcount : std_logic_vector(9 downto 0);  -- Vertical position (0-524)
  signal EndOfLine, EndOfField : std_logic;

  signal vga_hblank, vga_hsync,
  	 vga_vblank, vga_vsync : std_logic;  -- Sync. signals

  signal rectangle_h, rectangle_v, rectangle : std_logic;  -- rectangle area

  signal dataToCurrent_a : std_logic_vector(31 downto 0);
  signal junkData 		 : std_logic_vector(31 downto 0);
  signal AddrToCurrent_a : std_logic_vector(10 downto 0) := "00000000000";
  signal AddrToCurrent_b : std_logic_vector(10 downto 0) := "00000000000";
  signal wrenToCurrent_a : std_logic := '0';
  signal qFromCurrent_a  : std_logic_vector(31 downto 0);
  signal qFromCurrent_b  : std_logic_vector(31 downto 0);

  signal dataToNext_a : std_logic_vector(31 downto 0);
  signal AddrToNext_a : std_logic_vector(10 downto 0) := "00000000000";
  signal AddrToNext_b : std_logic_vector(10 downto 0) := "00000000000";
  signal wrenToNext_a : std_logic := '0';
  signal qFromNext_a  : std_logic_vector(31 downto 0);
  signal qFromNext_b  : std_logic_vector(31 downto 0);

  signal displayRate 	  : std_logic_vector(17 downto 0) := "000000000000000000";

  -- game logic related
  signal sr_counter             : 		std_logic_vector(3 downto 0) := "0000";
  signal sr_pos			: 		integer := 33;
  SIGNAL countNeighbors         :		STD_LOGIC_VECTOR(3 downto 0) := "0000";
  --SIGNAL currentPositionInOutput	:		STD_LOGIC_VECTOR(4 downto 0) := "11111";
  signal currentPositionInOutput :              integer := 31;
  signal beginningRow			:      	std_logic := '1';
  signal rowStartCount                  :	std_logic_vector(1 downto 0) := "00";

  signal pass : std_logic := '1';
  signal pass_counter : std_logic_vector(3 downto 0) := "0000";
 
  SIGNAL dataOut1,dataOut2,dataOut3	:	STD_LOGIC_VECTOR(33 downto 0) := "0000000000000000000000000000000000";
  SIGNAL outRegister				:	STD_LOGIC_VECTOR(31 downto 0) := "00000000000000000000000000000000";

  signal address : std_logic_vector(10 downto 0) := "00000000000";

  --signal bitPos_ram : std_logic_vector(4 downto 0) := "11111";
  signal hexToHEX   		: std_logic_vector(6 downto 0) := "1111111";
  signal bitPos 			: std_logic_vector(4 downto 0) := "11111";
  signal VGAReg 			: std_logic_vector(31 downto 0);
  signal videoBit 			: std_logic := '0';
  signal junkBit			: std_logic := '0';
  signal load_new 			: std_logic := '1';

  --signal first_pass : std_logic := '1';
  signal update : std_logic := '0';
  signal swap : std_logic := '0';
  signal done_update : std_logic := '0';
  signal load_next : std_logic := '0';

  signal loadData_a, loadData_b, loadq_a, loadq_b : std_logic_vector(31 downto 0);
  signal loadAddr_a, loadAddr_b : std_logic_vector(10 downto 0) := "00000000000";
  signal loadwren_a, loadwren_b : std_logic := '0';

  signal ram_address : std_logic_vector(10 downto 0) := "00000000000";
  signal load_address : std_logic_vector(10 downto 0) := "00000000000";
  signal ram_loaded : std_logic := '0';
  signal loaded : std_logic := '0';
  signal loadDelay : std_logic_vector(1 downto 0) := "00";
  
begin

  loadRAM : board_mem PORT MAP
	(
	  clock => clk25_diff,
	  data_a => loadData_a,
	  data_b => loadData_b,
	  address_a => loadAddr_a,
	  address_b => loadAddr_b,
	  wren_a => loadwren_a,
	  wren_b => loadwren_b,
	  q_a => loadq_a,
	  q_b => loadq_b
	);  

  currentRAM : board_mem PORT MAP
    (
      clock => clk25_diff,
      data_a => dataToCurrent_a,
      data_b => junkData,
      address_a => AddrToCurrent_a,
      address_b => AddrToCurrent_b,
      wren_a => wrenToCurrent_a,
      wren_b => junkBit,
      q_a => qFromCurrent_a,
      q_b => qFromCurrent_b
      );
  
  nextRAM : board_mem PORT MAP
    (
      clock => clk25_diff,
      data_a => dataToNext_a,
      data_b => junkData,
      address_a => AddrToNext_a,
      address_b => AddrToNext_b,
      wren_a => wrenToNext_a,
      wren_b => junkBit,
      q_a => qFromNext_a,
      q_b => qFromNext_b
      );

  -- Horizontal and vertical counters

  load : process (avs_s1_clk)
  begin
	if avs_s1_clk'event and avs_s1_clk = '1' and ram_loaded = '0' then
		if avs_s1_chipselect = '1' then
			if avs_s1_write = '1' then
				loadwren_a <= '1';
				loadAddr_a <= avs_s1_address;
				loadData_a <= avs_s1_writedata;
				if avs_s1_address = "11111111111" then
				  ram_loaded <= '1';
				end if;
			end if;
		end if;		
	end if;
  end process load;

  HCounter : process (clk25_diff, reset)
  begin    
    if reset = '1' then
      Hcount <= (others => '0');
    elsif clk25_diff'event and clk25_diff = '1' then
      if EndOfLine = '1' then
        Hcount <= (others => '0');
        Hpixel_pos <= "00000000000";
      else
        if Hcount = HSYNC + HBACK_PORCH + 192 + Hpixel_pos then
          Hpixel_pos <= (Hpixel_pos + 1);
        end if;
        Hcount <= Hcount + 1;
      end if;
    end if;
  end process HCounter;

  EndOfLine <= '1' when Hcount = HTOTAL - 1 else '0';
 
  VCounter: process (clk25_diff, reset)
  begin
    if reset = '1' then
      Vcount <= (others => '0');
    elsif clk25_diff'event and clk25_diff = '1' then
      if EndOfLine = '1' then
        if EndOfField = '1' then
          Vcount <= (others => '0');
          Vpixel_pos <= "00000000000";
        else
          if VCount = VSYNC + VBACK_PORCH + 112 + Vpixel_pos then
            Vpixel_pos <= (Vpixel_pos + 1);
          end if;
          Vcount <= Vcount + 1;
        end if;
      end if;
    end if;
  end process VCounter;

  EndOfField <= '1' when Vcount = VTOTAL - 1 else '0';

  -- State machines to generate HSYNC, VSYNC, HBLANK, and VBLANK

  HSyncGen : process (clk25_diff, reset)
  begin    
    if reset = '1' then
      vga_hsync <= '1';
    elsif clk25_diff'event and clk25_diff = '1' then
      if EndOfLine = '1' then
        vga_hsync <= '1';
      elsif Hcount = HSYNC - 1 then
        vga_hsync <= '0';
      end if;      
    end if;
  end process HSyncGen;
  
  HBlankGen : process (clk25_diff, reset)
  begin    
    if reset = '1' then
      vga_hblank <= '1';
    elsif clk25_diff'event and clk25_diff = '1' then
      if Hcount = HSYNC + HBACK_PORCH then
        vga_hblank <= '0';
      elsif Hcount = HSYNC + HBACK_PORCH + HACTIVE then
        vga_hblank <= '1';
      end if;      
    end if;
  end process HBlankGen;

  VSyncGen : process (clk25_diff, reset)
  begin    
    if reset = '1' then
      vga_vsync <= '1';
    elsif clk25_diff'event and clk25_diff = '1' then
      if EndOfLine ='1' then
        if EndOfField = '1' then
          vga_vsync <= '1';
        elsif Vcount = VSYNC - 1 then
          vga_vsync <= '0';
        end if;
      end if;      
    end if;
  end process VSyncGen;

  VBlankGen : process (clk25_diff, reset)
  begin    
    if reset = '1' then
      vga_vblank <= '1';
    elsif clk25_diff'event and clk25_diff = '1' then
      if EndOfLine = '1' then
        if Vcount = VSYNC + VBACK_PORCH - 1 then
          vga_vblank <= '0';
        elsif Vcount = VSYNC + VBACK_PORCH + VACTIVE - 1 then
          vga_vblank <= '1';
        end if;
      end if;      
    end if;
  end process VBlankGen;

  updater : process (clk25_diff)
  begin
    if clk25_diff'event and clk25_diff = '1' then
      if update = '1' and done_update = '0' then
		if ram_loaded = '1' and loaded = '0' then
		  if load_address = "11111111111" then
			loaded <= '1';
		  elsif loadDelay = "00" then
			wrenToCurrent_a <= '1';
			AddrToCurrent_a <= load_address;
			loadAddr_b <= load_address;
		  elsif loadDelay = "11" then
			dataToCurrent_a <= loadq_b;
			load_address <= (load_address + 1);
		  end if;
		  loadDelay <= (loadDelay + 1);
        elsif swap = '0' then 
          if(load_new = '1') then
            -- initial load
            if sr_counter = "0000" then
              address <= AddrToCurrent_a;
              if AddrToCurrent_a = "11111111000" then
                dataToNext_a <= outRegister;
                wrenToNext_a <= '1';
                AddrTonext_a <= "11111111111";
                AddrToCurrent_a <= "00000000000";
                done_update <= '1';
                dataOut1 <= (others => '0');
                dataOut2 <= (others => '0');
                dataOut3 <= (others => '0');
                outRegister <= (others => '0');
                sr_counter <= "1111";
              elsif AddrToCurrent_a(3 downto 0) = "0000" or AddrToCurrent_a(3 downto 0) = "1000" then
                beginningRow <= '1';
                wrenToNext_a <= '1';
                dataToNext_a <= outRegister;
                AddrToNext_a <= (AddrToCurrent_a + "111");
              end if;
            elsif(sr_counter = "0011") then
              dataOut1(31 downto 0) <= qFromCurrent_a;
            elsif(sr_counter = "0100") then
              AddrToCurrent_a <= (address + "1000");
            elsif(sr_counter = "0111") then
              dataOut2(31 downto 0) <= qFromCurrent_a;
            elsif(sr_counter = "1000") then
              AddrToCurrent_a <= (address + "10000");
            elsif(sr_counter = "1011") then
              dataOut3(31 downto 0) <= qFromCurrent_a;
              AddrToCurrent_a <= address;
              load_new <= '0';
              wrenToNext_a <= '0';
              if beginningRow = '1' then
                sr_pos <= 31;
              else
                sr_pos <= 33;
              end if;
            end if;
            sr_counter <= (sr_counter + 1);
          else            
            if sr_pos = 32 and beginningRow = '0' and (currentPositionInOutput = 0 or currentPositionInOutput = -1) then
              wrenToNext_a <= '1';
              dataToNext_a <= outRegister;
              AddrToNext_a <= (AddrToCurrent_a + "111");
              pass <= '1';
              currentPositionInOutput <= 31;
			  outRegister <= (others => '0');
            elsif sr_pos = 31 and currentPositionInOutput = 0 then
              pass <= '1';
              currentPositionInOutput <= 31;
            elsif (sr_pos > 1) and pass_counter < 8 then
             if (dataOut1(sr_pos) = '1' and pass_counter = 0) then
               countNeighbors <= (countNeighbors + dataOut1(sr_pos));
             end if;
             if (dataOut1(sr_pos-1) = '1' and pass_counter = 1) then
               countNeighbors <= (countNeighbors + dataOut1(sr_pos-1));
             end if;
             if dataOut1(sr_pos-2) = '1' and pass_counter = 2 then
               countNeighbors <= (countNeighbors + dataOut1(sr_pos-2));
             end if;
             if dataOut2(sr_pos) = '1' and pass_counter = 3 then
               countNeighbors <= (countNeighbors + dataOut2(sr_pos));
             end if;
             if dataOut2(sr_pos-2) = '1'and pass_counter = 4 then
               countNeighbors <= (countNeighbors + dataOut2(sr_pos-2));
             end if;
             if dataOut3(sr_pos) = '1' and pass_counter = 5 then
               countNeighbors <= (countNeighbors + dataOut3(sr_pos));
             end if;
             if dataOut3(sr_pos-1) = '1' and pass_counter = 6 then
               countNeighbors <= (countNeighbors + dataOut3(sr_pos-1));
             end if;
             if dataOut3(sr_pos-2) = '1' and pass_counter = 7 then
               countNeighbors <= (countNeighbors + dataOut3(sr_pos-2));
             end if;
             pass <= '0';
             pass_counter <= (pass_counter + 1);
            elsif pass = '0' then
              if dataOut2(sr_pos-1) = '1' then
                countNeighbors <= (countNeighbors + 1);
                if countNeighbors = "0010" or countNeighbors = "0011" then
                  if beginningRow = '1' then
                    outRegister(sr_pos-1) <= '1';
                  else
                    outRegister(currentPositionInOutput) <= '1';
                  end if;
                else
                  if beginningRow = '1' then
                    outRegister(sr_pos-1) <= '0';
                  else
                    outRegister(currentPositionInOutput) <= '0';
                  end if;
                end if;
              else
                if countNeighbors = "0011" then
                  if beginningRow = '1' then
                    outRegister(sr_pos-1) <= '1';
                  else
                    outRegister(currentPositionInOutput) <= '1';
                  end if;
                else
                  if beginningRow = '1' then
                    outRegister(sr_pos-1) <= '0';
                  else
                    outRegister(currentPositionInOutput) <= '0';
                  end if;
                end if;
              end if;
              if sr_pos = 2 then
                wrenToNext_a <= '0';
              end if;
              countNeighbors <= "0000";
              currentPositionInOutput <= (currentPositionInOutput - 1);
              pass_counter <= "0000";
              sr_pos <= (sr_pos - 1);            
              pass <= '1';
            else
			  if beginningRow = '0' then
				outRegister(31) <= '0';
			  end if;
              dataOut1(33 downto 32) <= dataOut1(1 downto 0);
              dataOut2(33 downto 32) <= dataOut2(1 downto 0);
              dataOut3(33 downto 32) <= dataOut3(1 downto 0);
              AddrToCurrent_a <= AddrToCurrent_a + 1;
              load_new <= '1';
              pass <= '1';
              address <= (others => '0');
              pass_counter <= "0000";
              beginningRow <= '0';
              sr_counter <= "0000";
              sr_pos <= 33;
              countNeighbors <= "0000";
            end if;
          end if;
        elsif swap = '1' then
          --swap = '1'		  	  
          if(load_new = '1') then
            -- initial load
            if sr_counter = "0000" then
              address <= AddrToNext_a;
              if AddrToNext_a = "11111111000" then
                dataToCurrent_a <= outRegister;
                wrenToCurrent_a <= '1';
                AddrToCurrent_a <= "11111111111";
                AddrToNext_a <= "00000000000";
                done_update <= '1';
                dataOut1 <= (others => '0');
                dataOut2 <= (others => '0');
                dataOut3 <= (others => '0');
                outRegister <= (others => '0');
                sr_counter <= "1111";
              elsif AddrToNext_a(3 downto 0) = "0000" or AddrToNext_a(3 downto 0) = "1000" then
                beginningRow <= '1';
                wrenToCurrent_a <= '1';
                dataToCurrent_a <= outRegister;
                AddrToCurrent_a <= (AddrToNext_a + "111");
              end if;
            elsif(sr_counter = "0011") then
              dataOut1(31 downto 0) <= qFromNext_a;
            elsif(sr_counter = "0100") then
              AddrToNext_a <= (address + "1000");
            elsif(sr_counter = "0111") then
              dataOut2(31 downto 0) <= qFromNext_a;
            elsif(sr_counter = "1000") then
              AddrToNext_a <= (address + "10000");
            elsif(sr_counter = "1011") then
              dataOut3(31 downto 0) <= qFromNext_a;
              AddrToNext_a <= address;
              load_new <= '0';
              wrenToCurrent_a <= '0';
              if beginningRow = '1' then
                sr_pos <= 31;
              else
                sr_pos <= 33;
              end if;
            end if;
            sr_counter <= (sr_counter + 1);
          else           
            if sr_pos = 32 and beginningRow = '0' and (currentPositionInOutput = 0 or currentPositionInOutput = -1) then
              wrenToCurrent_a <= '1';
              dataToCurrent_a <= outRegister;
              AddrToCurrent_a <= (AddrToNext_a + "111");
              pass <= '1';
			  outRegister <= (others => '0');
              currentPositionInOutput <= 31;
            elsif sr_pos = 31 and currentPositionInOutput = 0 then
              pass <= '1';
              currentPositionInOutput <= 31;
            elsif (sr_pos > 1) and pass_counter < 8 then
             if (dataOut1(sr_pos) = '1' and pass_counter = 0) then
               countNeighbors <= (countNeighbors + dataOut1(sr_pos));
             end if;
             if (dataOut1(sr_pos-1) = '1' and pass_counter = 1) then
               countNeighbors <= (countNeighbors + dataOut1(sr_pos-1));
             end if;
             if dataOut1(sr_pos-2) = '1' and pass_counter = 2 then
               countNeighbors <= (countNeighbors + dataOut1(sr_pos-2));
             end if;
             if dataOut2(sr_pos) = '1' and pass_counter = 3 then
               countNeighbors <= (countNeighbors + dataOut2(sr_pos));
             end if;
             if dataOut2(sr_pos-2) = '1'and pass_counter = 4 then
               countNeighbors <= (countNeighbors + dataOut2(sr_pos-2));
             end if;
             if dataOut3(sr_pos) = '1' and pass_counter = 5 then
               countNeighbors <= (countNeighbors + dataOut3(sr_pos));
             end if;
             if dataOut3(sr_pos-1) = '1' and pass_counter = 6 then
               countNeighbors <= (countNeighbors + dataOut3(sr_pos-1));
             end if;
             if dataOut3(sr_pos-2) = '1' and pass_counter = 7 then
               countNeighbors <= (countNeighbors + dataOut3(sr_pos-2));
             end if;
             pass <= '0';
             pass_counter <= (pass_counter + 1);
            elsif pass = '0' then
              if dataOut2(sr_pos-1) = '1' then
                countNeighbors <= (countNeighbors + 1);
                if countNeighbors = "0010" or countNeighbors = "0011" then
                  if beginningRow = '1' then
                    outRegister(sr_pos-1) <= '1';
                  else
                    outRegister(currentPositionInOutput) <= '1';
                  end if;
                else
                  if beginningRow = '1' then
                    outRegister(sr_pos-1) <= '0';
                  else
                    outRegister(currentPositionInOutput) <= '0';
                  end if;
                end if;
              else
                if countNeighbors = "0011" then
                  if beginningRow = '1' then
                    outRegister(sr_pos-1) <= '1';
                  else
                    outRegister(currentPositionInOutput) <= '1';
                  end if;
                else
                  if beginningRow = '1' then
                    outRegister(sr_pos-1) <= '0';
                  else
                    outRegister(currentPositionInOutput) <= '0';
                  end if;
                end if;
              end if;
              if sr_pos = 2 then
                wrenToCurrent_a <= '0';
              end if;
              countNeighbors <= "0000";
              currentPositionInOutput <= (currentPositionInOutput - 1);
              pass_counter <= "0000";
              sr_pos <= (sr_pos - 1);            
              pass <= '1';
            else
			  if beginningRow = '0' then
				outRegister(31) <= '0';
			  end if;
              dataOut1(33 downto 32) <= dataOut1(1 downto 0);
              dataOut2(33 downto 32) <= dataOut2(1 downto 0);
              dataOut3(33 downto 32) <= dataOut3(1 downto 0);
              AddrToNext_a <= AddrToNext_a + 1;
              load_new <= '1';
              pass <= '1';
              address <= (others => '0');
              pass_counter <= "0000";
              beginningRow <= '0';
              sr_counter <= "0000";
              sr_pos <= 33;
              countNeighbors <= "0000";
            end if;
          end if;
		end if;
	  elsif update = '0' and done_update = '1' then
        dataOut1 <= "0000000000000000000000000000000000";
        dataOut2 <= "0000000000000000000000000000000000";
        dataOut3 <= "0000000000000000000000000000000000";
        outRegister <= (others => '0');
        address <= (others => '0');
        sr_counter <= "0000";
        wrenToNext_a <= '0';
        wrenToCurrent_a <= '0';
        AddrToNext_a <= "00000000000";
        AddrToCurrent_a <= "00000000000";
        done_update <= '0';
        hexToHEX <= (hexToHEX + 1);
        swap <= (not swap);
      end if;
    end if;
  end process updater;  
               
  HEX <= hexToHEX;

  -- Rectangle generator

  RectangleHGen : process (clk25_diff, reset)
  begin
    if reset = '1' then
      rectangle_h <= '1';
    elsif clk25_diff'event and clk25_diff = '1' then
	  if loaded = '0' then
	  	rectangle_h <= '0';
      elsif swap = '0' then
        if (Vcount = VSYNC + VBACK_PORCH - 1 + 111) then
          if done_update = '1' then
            update <= '0';                      
          end if;
          rectangle_h <= '0';
          VGAReg <= qFromCurrent_b;
          bitPos <= "11111";
        elsif (Vcount = VSYNC + VBACK_PORCH - 1 + 369) then
          rectangle_h <= '0';
          AddrToCurrent_b <= "00000000000";
          displayRate <= (displayRate + 1);
          if(displayRate = "00000000111111111") then
            displayRate <= "000000000000000000";
            update <= '1';
          end if;
        elsif (Hcount > HSYNC + HBACK_PORCH + 191) and (Hcount < HSYNC + HBACK_PORCH + 449) and
          (Vcount > VSYNC + VBACK_PORCH - 1 + 111) and (Vcount < VSYNC + VBACK_PORCH - 1 + 368) then
          videoBit <= VGAReg(conv_integer(bitPos));
          if (videoBit = '1') then
            rectangle_h <= '1';
          else
            rectangle_h <= '0';
          end if;
          
          if (bitPos = "10000") then
            AddrToCurrent_b <= (AddrToCurrent_b + 1);
          elsif bitPos= "00000" then
            VGAReg <= qFromCurrent_b;
          end if;
          bitPos <= (bitPos - 1);
        else
          bitPos <= "11111";
        --VGAReg <= qFromCurrent_b;
          rectangle_h <= '0';
        end if;
      elsif swap = '1' then
        -- swap = '1'
        if (Vcount = VSYNC + VBACK_PORCH - 1 + 111) then 
          update <= '0';
          rectangle_h <= '0';
          VGAReg <= qFromNext_b;
          bitPos <= "11111";
        elsif (Vcount = VSYNC + VBACK_PORCH - 1 + 369) then
          rectangle_h <= '0';
          AddrToNext_b <= "00000000000";
          displayRate <= (displayRate + 1);
          if(displayRate = "00000000011111111") then
            displayRate <= "000000000000000000";
            --hexToHEX <= (hexToHEX + 1);
            update <= '1';
          end if;
        elsif (Hcount > HSYNC + HBACK_PORCH + 191) and (Hcount < HSYNC + HBACK_PORCH + 449) and
          (Vcount > VSYNC + VBACK_PORCH - 1 + 111) and (Vcount < VSYNC + VBACK_PORCH - 1 + 368) then
          videoBit <= VGAReg(conv_integer(bitPos));
          if (videoBit = '1') then
            rectangle_h <= '1';
          else
            rectangle_h <= '0';
          end if;
          
          if (bitPos = "10000") then
            AddrToNext_b <= (AddrToNext_b + 1);
          elsif bitPos= "00000" then
            VGAReg <= qFromNext_b;
          end if;
          bitPos <= (bitPos - 1);
        else
          bitPos <= "11111";
          --VGAReg <= qFromNext_b;
          rectangle_h <= '0';
        end if;
      end if;
    end if;
  end process RectangleHGen;

  RectangleVGen : process (clk25_diff, reset)
  begin
    if reset = '1' then
      rectangle_v <= '0';
    elsif clk25_diff'event and clk25_diff = '1' then
      if loaded = '0' then
		rectangle_v <= '0';
      elsif EndOfLine = '1' then
        if (Vcount > VSYNC + VBACK_PORCH - 1 + 367) or (Vcount < VSYNC + VBACK_PORCH - 1 + 112) then
          rectangle_v <= '0';
        else
		  rectangle_v <= '1';		
        end if;
      end if;      
    end if;
  end process RectangleVGen;

  rectangle <= rectangle_h and rectangle_v;

  -- Registered video signals going to the video DAC

  VideoOut: process (clk25_diff, reset)
  begin
    if reset = '1' then
      VGA_R <= "0000000000";
      VGA_G <= "0000000000";
      VGA_B <= "0000000000";
    elsif clk25_diff'event and clk25_diff = '1' then
      if rectangle = '1' then
        VGA_R <= "1111111111";
        VGA_G <= "1111111111";
        VGA_B <= "1111111111";
      elsif vga_hblank = '0' and vga_vblank ='0' then
        VGA_R <= "0000000000";
        VGA_G <= "0000000000";
        VGA_B <= "1111111111";
      else
        VGA_R <= "0000000000";
        VGA_G <= "0000000000";
        VGA_B <= "0000000000";    
      end if;
    end if;
  end process VideoOut;

  VGA_CLK <= clk25_diff;
  VGA_HS <= not vga_hsync;
  VGA_VS <= not vga_vsync;
  VGA_SYNC <= '0';
  VGA_BLANK <= not (vga_hsync or vga_vsync);

end rtl;
